@@ -1,10 +1,10 @@
 use strict;
 use warnings;
-package Test::JSON::Schema::Acceptance; # git description: v1.001-8-gf2f5951
+package Test::JSON::Schema::Acceptance; # git description: v1.003-3-g266ec84
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Acceptance testing for JSON-Schema based validators like JSON::Schema
 
-our $VERSION = '1.002';
+our $VERSION = '1.004';
 
 use 5.014;
 no if "$]" >= 5.031009, feature => 'indirect';
@@ -59,6 +59,14 @@ has include_optional => (
   default => 0,
 );
 
+has skip_dir => (
+  is => 'ro',
+  isa => ArrayRef[Str],
+  coerce => sub { ref($_[0]) ? $_[0] : [ $_[0] ] },
+  lazy => 1,
+  default => sub { [] },
+);
+
 has results => (
   is => 'rwp',
   init_arg => undef,
@@ -208,7 +216,6 @@ sub _run_test {
 
   Test2::API::run_subtest($test_name,
     sub {
-      my $ctx = Test2::API::context;
       my ($result, $schema_before, $data_before, $schema_after, $data_after);
       try {
         {
@@ -229,15 +236,17 @@ sub _run_test {
             $test_group->{schema}, $test->{data};
         }
 
+        my $ctx = Test2::API::context;
+
         # skip the ugly matrix comparison
+        my $expected = $test->{valid} ? 'true' : 'false';
         if ($result xor $test->{valid}) {
           my $got = $result ? 'true' : 'false';
-          my $expected = $test->{valid} ? 'true' : 'false';
           $ctx->fail('test failed', 'expected '.$expected.'; got '.$got);
           $pass = 0;
         }
         else {
-          $ctx->ok(1, 'test passes');
+          $ctx->ok(1, 'test passes: data is valid: '.$expected);
           $pass = 1;
         }
 
@@ -245,13 +254,15 @@ sub _run_test {
           if $data_before ne $data_after;
         $pass &&= Test2::Tools::Compare::is($schema_after, $schema_before, 'evaluator did not mutate schema')
           if $schema_before ne $schema_after;
+
+        $ctx->release;
       }
       catch {
         chomp(my $exception = $_);
+        my $ctx = Test2::API::context;
         $ctx->fail('died: '.$exception);
+        $ctx->release;
       };
-
-      $ctx->release;
     },
     { buffered => 1, inherit_trace => 1 },
   );
@@ -296,6 +307,7 @@ sub _build__test_data {
   $self->test_dir->visit(
     sub {
       my ($path) = @_;
+      return if any { $self->test_dir->child($_)->subsumes($path) } @{ $self->skip_dir };
       return if not $path->is_file;
       return if $path !~ /\.json$/;
       my $data = $self->_json_decoder->decode($path->slurp_raw);
@@ -335,7 +347,7 @@ Test::JSON::Schema::Acceptance - Acceptance testing for JSON-Schema based valida
 
 =head1 VERSION
 
-version 1.002
+version 1.004
 
 =head1 SYNOPSIS
 
@@ -467,6 +479,13 @@ during C<make test> or C<prove>.
 
 Optional. When true, tests in subdirectories (most notably F<optional/> are also included.
 
+=head2 skip_dir
+
+Optional. Pass a string or arrayref consisting of relative path name(s) to indicate directories
+(within the test directory as specified above with C<specification> or C<test_dir>) which will be
+skipped. Note that this is only useful currently with C<include_optional => 1>, as otherwise all
+subdirectories would be skipped anyway.
+
 =head1 SUBROUTINES/METHODS
 
 =head2 acceptance
